@@ -6,6 +6,8 @@ library(dplyr)
 library(ggplot2)
 library(BiocParallel)
 
+run_on_platform <- (Sys.info()['sysname'] == "Linux")
+
 ## ----create data, echo=TRUE, warning=FALSE------------------------------------
 m1 <- matrix(rep(1:20000, each = 100), ncol = 20000, byrow = FALSE)
 ex_file <- tempfile(fileext = ".h5")
@@ -35,14 +37,15 @@ system.time(
 )
 identical(res2, res3)
 
-## ----singleReads, cache = TRUE------------------------------------------------
-columns <- sample(x = seq_len(20000), size = 10000, replace = FALSE) %>%
+## ----chooseColunns, eval = TRUE-----------------------------------------------
+columns <- sample(x = seq_len(20000), size = 1000, replace = FALSE) %>%
   sort()
-
 f1 <- function(cols, name) { 
   h5read(file = ex_file, name = name, 
          index = list(NULL, cols))
   }
+
+## ----singleReads, eval = run_on_platform, cache = TRUE------------------------
 system.time(res4 <- vapply(X = columns, FUN = f1, 
                            FUN.VALUE = integer(length = 100), 
                            name = 'counts'))
@@ -86,7 +89,7 @@ ggplot(bm2, aes(x = block_size, y = time)) +
   theme_bw() + 
   ylab('time (seconds)')
 
-## ----eval = TRUE, echo = FALSE, fig.width=6, fig.height=3, fig.wide = TRUE, fig.cap='The time taken to join hyperslabs increases expontentially with the number of join operations.  These timings are taken with no reading occuring, just the creation of a dataset selection.'----
+## ----hyperslab-benchmark, eval = run_on_platform, echo = FALSE, fig.width=6, fig.height=3, fig.wide = TRUE, fig.cap='The time taken to join hyperslabs increases expontentially with the number of join operations.  These timings are taken with no reading occuring, just the creation of a dataset selection.'----
 ## this code demonstrates the exponential increase in time as the 
 ## number of hyberslab unions increases
 
@@ -220,7 +223,7 @@ split_and_gather <- function(output_file, input_dsets, BPPARAM = NULL) {
 #  split_and_gather(tempfile(), input_dsets = dsets,
 #                   BPPARAM = MulticoreParam(workers = 2))
 
-## ----run-writing-benchmark, cache = TRUE, message=FALSE, echo=FALSE-----------
+## ----run-writing-benchmark, eval = run_on_platform, cache = TRUE, message=FALSE, echo=FALSE----
 bench_results <- bench::mark(
   "simple writer" = simple_writer(file_name = tempfile(), dsets = dsets),
   "split/gather - 1 core" = split_and_gather(tempfile(), input_dsets = dsets, 
@@ -229,10 +232,13 @@ bench_results <- bench::mark(
                                               BPPARAM = MulticoreParam(workers = 2)), 
   "split/gather - 4 cores" = split_and_gather(tempfile(), input_dsets = dsets, 
                                               BPPARAM = MulticoreParam(workers = 4)),
-  iterations = 3, check = FALSE, time_unit = "s", memory = FALSE, filter_gc = FALSE
+  iterations = 2, check = FALSE, time_unit = "s", memory = FALSE, filter_gc = FALSE
 )
 bench_results |> select(expression, min, median)
 
+## ----fake-timings, eval = !run_on_platform, message = FALSE, echo = FALSE-----
+#    bench_results <- data.frame(median = 4:1)
+
 ## ----sessionInfo, echo=FALSE--------------------------------------------------
 sessionInfo()
 
