@@ -52,7 +52,7 @@ for (int li = 0, lj = 0; li < LENGTH(FROM); li++) {               \
 }                                                                 \
 FROM = to;                                                        \
 } while(0)
-
+  
 /* hid_t H5Dcreate( hid_t loc_id, const char *name, hid_t dtype_id, hid_t space_id, hid_t lcpl_id, hid_t dcpl_id, hid_t dapl_id ) */
 SEXP _H5Dcreate( SEXP _loc_id, SEXP _name, SEXP _dtype_id, SEXP _space_id, SEXP _lcpl_id, SEXP _dcpl_id, SEXP _dapl_id ) {
 
@@ -672,16 +672,59 @@ SEXP H5Dread_helper_ARRAY(hid_t dataset_id, hid_t file_space_id, hid_t mem_space
     
 }
 
+/* test whether this looks like it's storing a complex datatype as a compound */
+int is_complex(hid_t dtype_id) {
+  
+  int res = 0;
+
+  if(H5Tget_nmembers(dtype_id) == 2) {
+    
+    char *field1 = H5Tget_member_name(dtype_id, 0);
+    char *field2 = H5Tget_member_name(dtype_id, 1);
+    
+    if((strcmp(field1, "r") == 0) && (strcmp(field2, "i") == 0))
+      res = 1;
+    
+    H5free_memory(field1);
+    H5free_memory(field2);
+  }
+  
+  return(res);
+}
+
+SEXP H5Dread_helper_COMPLEX(hid_t dataset_id, hid_t file_space_id, hid_t mem_space_id, hsize_t n, SEXP Rdim,
+                          hid_t dtype_id, int native) {
+
+  SEXP Rval;
+  PROTECT(Rval = allocVector(CPLXSXP, n));
+  void * buf = COMPLEX(Rval);
+  
+  herr_t herr = H5Dread(dataset_id, dtype_id, mem_space_id, file_space_id, H5P_DEFAULT, buf );
+  if(herr < 0) {
+    UNPROTECT(1);
+    error("Unable to read dataset");
+  }
+
+  if (native) {
+    PERMUTE(Rval, COMPLEX, mem_space_id);
+  }
+
+  setAttrib(Rval, R_DimSymbol, Rdim);
+  UNPROTECT(1 + native);
+  return Rval;
+}
+
 SEXP H5Dread_helper_COMPOUND(hid_t dataset_id, hid_t file_space_id, hid_t mem_space_id, hsize_t n, SEXP Rdim, SEXP _buf, 
                              hid_t dtype_id, hid_t cpdType, int cpdNField, char ** cpdField, int compoundAsDataFrame,
                              int bit64conversion, int native ) {
+  
+    SEXP Rval;
 
     if ((LENGTH(Rdim) > 1) && compoundAsDataFrame) {
         compoundAsDataFrame = 0;
         warning("Cannot coerce multi-dimensional data to data.frame. Data returned as a list.");
     }
-    
-    SEXP Rval;
+
     if (cpdType < 0) {
         int N = H5Tget_nmembers(dtype_id);
         PROTECT(Rval = allocVector(VECSXP, N));
@@ -710,7 +753,7 @@ SEXP H5Dread_helper_COMPOUND(hid_t dataset_id, hid_t file_space_id, hid_t mem_sp
             SEXP rn = PROTECT(allocVector(INTSXP, INTEGER(Rdim)[0]));
             for (int i=0; i<INTEGER(Rdim)[0]; i++) { INTEGER(rn)[i] = i+1; }
             UNPROTECT(1);
-            setAttrib(Rval, mkString("row.names"), rn);
+            setAttrib(Rval, R_RowNamesSymbol, rn);
             setAttrib(Rval, R_ClassSymbol, mkString("data.frame"));
         }
         UNPROTECT(2);
@@ -737,6 +780,23 @@ SEXP H5Dread_helper_COMPOUND(hid_t dataset_id, hid_t file_space_id, hid_t mem_sp
     return(Rval);
 }
 
+SEXP H5Dread_helper_COMPOUND_OR_COMPLEX(
+    hid_t dataset_id, hid_t file_space_id, hid_t mem_space_id, hsize_t n, SEXP Rdim, SEXP _buf, 
+    hid_t dtype_id, hid_t cpdType, int cpdNField, char ** cpdField, int compoundAsDataFrame,
+    int bit64conversion, int native ) {
+  
+  SEXP Rval;
+  
+  if(is_complex(dtype_id)) {
+    Rval = H5Dread_helper_COMPLEX(dataset_id, file_space_id, mem_space_id, n, Rdim, dtype_id, native);
+  } else {
+    Rval = H5Dread_helper_COMPOUND(dataset_id, file_space_id, mem_space_id, n, Rdim, _buf, 
+                                   dtype_id, cpdType, cpdNField, cpdField, compoundAsDataFrame,
+                                   bit64conversion, native);
+  }
+  
+  return(Rval);
+}
 
 //SEXP H5Dread_helper_REFERENCE(hid_t attr_id, hsize_t n, SEXP Rdim, SEXP _buf, hid_t dtype_id) {
 SEXP H5Dread_helper_REFERENCE(hid_t dataset_id, hid_t file_space_id, hid_t mem_space_id, hsize_t n, SEXP Rdim, SEXP _buf,
@@ -801,7 +861,7 @@ SEXP H5Dread_helper(hid_t dataset_id, hid_t file_space_id, hid_t mem_space_id, h
                                      dtype_id, cpdType, cpdNField, cpdField, compoundAsDataFrame, native );
     } break;
     case H5T_COMPOUND: {
-        Rval = H5Dread_helper_COMPOUND(dataset_id, file_space_id, mem_space_id, n, Rdim, _buf, 
+        Rval = H5Dread_helper_COMPOUND_OR_COMPLEX(dataset_id, file_space_id, mem_space_id, n, Rdim, _buf, 
                                        dtype_id, cpdType, cpdNField, cpdField, compoundAsDataFrame,
                                        bit64conversion, native );
     } break;
@@ -1021,6 +1081,16 @@ SEXP _H5Dwrite( SEXP _dataset_id, SEXP _buf, SEXP _file_space_id, SEXP _mem_spac
 
         buf = read_string_datatype(mem_type_id, _buf);
 
+        break;
+    case CPLXSXP :
+        if(isNull(_mem_type_id)) {
+          mem_type_id = H5Dget_type( dataset_id );
+        } else {
+          mem_type_id = STRSXP_2_HID( _mem_type_id );
+        } 
+        if (native)
+          PERMUTE(_buf, COMPLEX, dim_space_id);
+        buf = COMPLEX(_buf);
         break;
     case S4SXP :
         if(R_check_class_etc(_buf, H5Ref) >= 0) {
